# -*- coding: utf-8 -*-

"""
=============================  SFLOW RENDER FILE  =============================
This is the master file for the Real Time Network Sonification @ Fraunhofer IIS
Proper versioning of this file is done by snapshot of the entire directory,
with a date or obvious numerical sequence.

The run process is always to
$cd that directory
$ python
>>> import _sflowLive

(Individual copies of just this file are sometimes taken during development
and named accordingly. They not production files)

Update: this file will become subsumed by the _netsonUI.py file, which will
include it.

Live streaming from one of these URLs:
audio to laptop: rtmp://kunststream.iis.fhg.de:1935/live/sonification.stream
sound & image to laptop: http://kunststream.iis.fhg.de:1935/live/sonification.stream/playlist.m3u8

rtsp://kunststream.iis.fhg.de:1935/live/sonification.stream
http://kunststream.iis.fhg.de:1935/live/sonification.stream/playlist.m3u8

#http://kunststream.iis.fhg.de:1935/live/sonification.stream/manifest.mpd
#http://1kunststream.iis.fhg.de:1935/live/sonification.stream/Manifest
=====++++++= NB  CONTROL SETTINGS ARE AT THE BOTTOM OF THIS FILE  =++++++======
"""

# import the other NetSon files:
from sflowReadClasses import *
from sflowOrchestraNew7 import *
from sflowPipeReadClass import *

"""
#  :::::::::::::::::::::  GLOBAL VARIABLE DEFINITIONS  :::::::::::::::::::::::
"""
# NB global variable declarations should all be here - not in included files
# dataDirectory is where the fifo and all non-python controls are for sflow, dta backup etc
dataDirectory="/Users/wor/sonipy/_CorpoReal/data/sflowCapture/"    # on mpb288
#dataDirectory="/Users/wor-local/sonipy/_CorpoReal_data/internet/sflowCapture/" # on mac200 and mac233

#  ----------------------- UPD TOL LOCAL MACHINE - FOR RUNNING THEVISUAL SW ON THIS MACHINE UNDER VPN
global USE_LOCAL_IP     ; USE_LOCAL_IP = True
global LOCAL_UDP_IP     ;  LOCAL_UDP_IP = "127.0.0.1"
global UDP_SEND_PORT    ; UDP_SEND_PORT = 4444

#  ----------------------- USER INTERFACE CONTROL ---------
UIcontrolURL = "http://www2.iis.fraunhofer.de/corpo/programm.txt"
#  the file updated by the web GUI control - read once each FIFO read

TRACE = False                    # for debut - still works!

global IFSTATS ; IFSTATS = False # DON'T USE: currently cause fatal crash with IOError: too many files open
global UI_CONTROL_LIST; UI_CONTROL_LIST = ['0', '0', '0', '0']  # [radioButNr, Checkbox1, checkbox2, checkbox3]
# NB UI_CONTROL_LIST is a local contol list and is used as an alternative to WEB_UI
# i.e. so that the web UI settings are simulated without the need for the web interdface itself

global WEB_UI ; WEB_UI = False
if WEB_UI == True:
    UI_CONTROL_LIST = ['0', '0', '0', '0']  # initializer to allow interface a clean go

global UIsettings ;
global FILE_UI ; FILE_UI = True # initializer to allow interface a clean go. NB used by  FILE_UI but not WEB_UI which is depricated
jsonUIfile="_UIcontrols.json"

global FILE_UI_FID ; FILE_UI_FID = 0

if FILE_UI == True:
    FILE_UI_FID = open (jsonUIfile, 'r')
    # initialize to silence
    UIsettings = {}
    UIsettings['a']=['a', "set All to unknown"]
    UIsettings['d']=['d', "Divisions"]
    UIsettings['e']=['e', "Erlangen"]
    UIsettings['f']=['f', "Fileservers"]
    UIsettings['i']=['i', "IT group"]
    UIsettings['l']=['l', "Load meter"]
    UIsettings['m']=['m', "THIS MENU"]
    UIsettings['p']=['p', "printers"]
    UIsettings['q']=['q', "QUIT"]
    UIsettings['r']=['r', "depARtments"]
    UIsettings['s']=['s', "Servers"]
    UIsettings['u']=['u', "source Unidentified"]
    UIsettings['v']=['v', "VoIP"]
    UIsettings['x']=['x', "EXECUTE"]
    UIsettings['z']=['z', "silenZZZe"]

print UIsettings

global LEGEND_IDENTITY_MASK ; LEGEND_IDENTITY_MASK = False  # if False, use the full ID of the IP Addr, else use the short form.
                                            # Both forms are defined in the config.properties file.
global OP_EVENT_COUNT   ; OP_EVENT_COUNT = 0 # unique sonification event number - incremented only when a sound event is generated
#global lastPitch      ; lastPitch = 60  # midi - used to test live flow

global NULL_IP_NAME     ; NULL_IP_NAME = "null"
"""
:::::::::::::::::::::    SOME GENERIC CONSTRUCTS  :::::::::::::::::::::::::::::
"""
"""
# try " ".rjust(90,':')
# scaling vectors - used to provide degress of variation of sonification parameters
reduceMultLgeRnge = [0.4, 0.6, 0.8, 1]  # values by which certain paramaters may be varied. for shaping
reduceMultMedRnge = [0.7, 0.8, 0.9, 1]  # values by which certain paramaters may be varied. for shaping
reduceMultSmlRnge = [0.85, 0.8, 0.95, 1]  # values by which certain paramaters may be varied. for shaping
expandMultLgeRnge = [1,1.2, 1.4, 1.6]  # values by which certain paramaters may be varied. for shaping
expandMultMedRnge = [1, 1.1, 1.2, 1.3 ]  # values by which certain paramaters may be varied. for shaping
expandMultSmlRnge = [1, 1.05, 1.1, 1.15]  # values by which certain paramaters may be varied. for shaping
rduceExpMultLgeRnge = [0.4, 0.6, 0.8, 1]  # values by which certain paramaters may be varied. for shaping
rduceMultMedRnge = [0.7, 0.8, 0.9, 1]  # values by which certain paramaters may be varied. for shaping
rduceMultSmlRnge = [0.8, 0.85, .9, 1]  # values by which certain paramaters may be varied. for shaping
"""
spaces2Right = 10 * " "
manyColons = (40 * " ") + (50 * ":") # for page RHS stick-outs
"""
#  :::::::::::::::::::::  IP_LOCATION_DISTANCE RESOLUTION :::::::::::::::::::::
"""
from math import sin, cos, sqrt, atan2, radians

homelat = radians (49.5500)     # degrees to radians # latitude of ErlangenFile Server = '153.96.171.0'
homelong = radians (11.0036)    # degrees to radians longitude of Erlangen
halfEquatorLength = 40075 / 2.0 # half the equator's length in km
meanEarthRadius =  6371.0       # in km

import socket, struct

def ip2long(ipString):
    return struct.unpack("!L", socket.inet_aton(ipString))[0]

iisIPLongInt = ip2long('153.96.171.0')
maxIPint = ip2long('255.255.255.255')
floatMaxIPint = float (maxIPint)

# put code here for GLOBAL DISTANCE RATIO CODE - USING A PUBLIC DATABASE - if/when it works -SEE EOF

def ipDistanceRatio(ipaddrString):
    # computes a ratio (0-1) ratio of similarily of ip address to the Erlangen File server
        ipLongInt = ip2long(ipaddrString)
        distanceRatio = (maxIPint-abs(iisIPLongInt-ipLongInt))/floatMaxIPint
        return distanceRatio
"""
# ::::::::::::::::::::::::::: GLOBAL TIME-WARPING :::::::::::::::::::::::::::::
"""
# see also sflowReadClasses.readSflowFile2RecordList()  which does the initial time-conversion and scaling
# in order to determine if the record is redundant or not. May be better to delay that decision to here in live stream as
# there is not an issue w. the length of the input Pipe array.
def timeScaler (eventList, nrSecs):
    """ Scales the eventList so it will last nrSecs duration on play. Used for NRT sonifications."""
    period = float(eventList[-1][2] - eventList[0][2]) # timeLastEvent -     timeFirstEvent
    if period == 0:
        return 1.0
    elif nrSecs == 0:
        return float(eventList[-1][2] - eventList[0][2])
    else:
        return float(period)/nrSecs
#    print "timing:", timeFirstEvent, timeLastEvent, timeLastEvent-timeFirstEvent

def arpeggiate(startTimeArray, deltaT=0):
    """
    Used to spread multiple exactly-simultaneous events by a factor of deltaT increments.
    # deltaT is increment in seconds  by which consecutive events that occur at the same time are arpeggiated.
    Does NOT push subsequent events out of their correct event time,
    so in dense periods will generate for simultaneous events. Default deltaT=0 implies no arpeggiation.
    """
    arpegiateQ = True
    while arpegiateQ:
        diffArray=np.ediff1d(startTimeArray)    # len fiffArray 1< startTimeArray, so
        if 0 in diffArray:                      # there are simultaneous, so offset them with deltaT from the last offset
            diffArray=np.insert(diffArray, 0, 1) # insert a 1 in the 0th position to make the arrays the same length
            invertDiffArray = diffArray==0     # swap 0s <> 1s into a new array. Relies in 0 being false
            startTimeArray = startTimeArray + (invertDiffArray * deltaT)
        else:                                   # the are not any simultaneous events
            arpegiateQ=False
    return startTimeArray

def note2cps(noteNum, fundFreq, tet):
    # converts a note number to frequency, given fundFreq = note 1 and tet is Nr of Equal tempered /8ve
    # to make it zero-based, comment out the next line
    noteNum -= 1
    return 0.5* fundFreq * (2**((noteNum)/ float (tet)))

def durationScaler (dur, freq, centreFreq):
    # unfinished
    # scales the duration so tht the higer the tone the shorter and vice versa
    # assumes a flat response at centreFreq
    freqRange = range (20, 4000)

"""
# :::::::::::      define the Instrument Dictionary - set which are active    :::::::::::::::::#
"""
# the instrument dictionary is curently just used to switch individual instruments in the mix on or off. Perhaps more funcrions later
instrumentNames = ['sineInstr_01', 'pluckInstr_02', 'preppno_03', 'fmb3_04', 'cheby_05', 'fmbell_06', 'fmvoice_07', 'highhat_08',
                   'glisInstr_09', 'fractalNoise_10', 'crunch_11', 'guiro_12', 'bamboo_13', 'tambourine_14', 'pulseFMsynth_15', 'testInstrument_20']

instrumentDict={}
instrumentIsActiveField = 1 # the 1str element of the array contains True/False for whether the instrument is active or not.
def makeInstrumentDict():
    # defines a dict of on-off switches for each instrument
    for instrName in instrumentNames:
                       # numericStr      nameString   active or not flag
        instrumentDict[instrName[-2:]] = [instrName, True]
makeInstrumentDict()

def printInstrumentDict():
    print "INSTRUMENTS IN USE\n__________________"
    print "InstrNr [Name,         True/False]"
    kees = instrumentDict.keys()
    kees.sort()
    for kee in kees:
        print kee, instrumentDict[kee]
printInstrumentDict()

#-------------
"""global LOCAL_UDP_SOCK ; LOCAL_UDP_SOCK = 0
def openUDPport(IPaddr, portNr):
    global LOCAL_UDP_SOCK
    LOCAL_UDP_SOCK = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # Address,  UDP

openUDPport(LOCAL_UDP_IP,UDP_SEND_PORT )

def sendUDP(*args):
    # emits UPD message and emits it
    global LOCAL_UDP_IP
    global UDP_SEND_PORT
    global LOCAL_UDP_SOCK
    message = ' '.join(map(str, args))
    print message
    LOCAL_UDP_SOCK.sendto(message, (LOCAL_UDP_IP, UDP_SEND_PORT))

"""
"""
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::#
# :::::::::::::::::::::::::::      playAllInstruments     :::::::::::::::::::::::::::::::::::::#
# ::::::::::::::::::::::      sonification switching logic     ::::::::::::::::::::::::::::::::#
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::#
"""
#import csnd6
import json
from random import random, choice

def makeCSevent (*args):
    global TRACE
    #Makes a csound event and plays it in an already active Performance thread
    eventStr = ' '.join(map(str, args))
    if TRACE:
        print spaces2Right + eventStr
#    csPerf.InputMessage(eventStr+'\n')
#    cs.ScoreEvent(eventStr)
    cs.ReadScore (eventStr) #    print event

import socket
import os
LOCAL_UDP_SOCK = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # Address,  UDP

def sendUDP(*args):
    # creates UPD message string and emits it. Used with MakeCSevent each time an event is emitted
#    print args
    message = ','.join(map(str, args))  # string of comma separated items
    message = message.replace('"','')   # o remove the quotes around the src and dsp names
    LOCAL_UDP_SOCK.sendto(message, (LOCAL_UDP_IP, UDP_SEND_PORT))


def playAllInstruments(durationRequired=0):
    # This function needs to be executed each time the playlist is updated from the live Pipe Feed.
    # Else, it is executed once after s.makePlayList()
    # perhaps it should take the playlist as an argument
    # ::::::::::: Update UI Controls :::::::::::::
    global IFSTATS
    global WEB_UI
    global FILE_UI
    global UI_CONTROL_LIST
#    global lastPitch
    global OP_EVENT_COUNT
    global LEGEND_IDENTITY_MASK
    global NULL_IP_NAME
#    print "    global LEGEND_IDENTITY_MASK: ", LEGEND_IDENTITY_MASK
    # === START-TIME ===
    if durationRequired <= 0:   # special case - play in real-time
        timeScaleFactor = 1
    else:
        timeScaleFactor = float(timeScaler (s.playList, durationRequired))
    if TRACE:
        print manyColons + "timeScaleFactor: ", timeScaleFactor
    startTimes=[]   # a list of the relative time offsets from the first event in each FIFO read
    for i in xrange(len(s.playList)):
        startTimes +=[(s.playList[i][2]-s.playList[0][2])/timeScaleFactor] # 0th is thus == 0 by default
    deltaBeat = 0.0618 # beat increment by which consecutive events that occur at the same time are arpeggiated
    if deltaBeat !=0:
        startTimes = arpeggiate(startTimes, deltaBeat)  #modify the startimes according to the arpeggiation rules

    # === FREQUENCY ===
    fundChoice = choice ([56.25, 60.0, 64])     # fundamental freq (AC mains AC); fund = 60.0
                                                # NB fund != fundChoice.
                                                # Fundchoice is used when we need to choose
                                                # from a group of fixed pitches, such as with Inst 7 for eg.
    fund = 60.0 # choice ([60.0, 64.0])

    scale1 = [420.0, 471.434, 529.167, 593.970, 666.708, 748.355, 840.0]  # whole tone scale on 7*60 Hz
    scale1a = [120.0, 134.695, 151.191, 169.706, 190.488, 213.816, 240.0] #  whole tone scale on 6*60 Hz
    scale2 = [444.974, 499.466, 560.632, 629.288, 706.352, 792.854, 889.948] #  whole tone scale on 7*60 Hz + sT
    scale2a = [127.135, 142.705, 160.181, 179.797, 201.815, 226.530, 254.271]  # whole tone scale on 6*60 Hz

 #   scale1 = [440.000, 493.883, 554.365, 622.254, 698.456, 783.991, 880.00]   # whole tone

    lastScaleNote = 0
    lastAttackTime = 0
    nrStreams = 8   #  half the number of azimuthations required. i.e. if nrStreams = 8, nrAzimuths = 16

    # === VIRTUAL FIXED LOCATIONS ===
    azimuths = [((180.0/nrStreams)* azimuth) for azimuth in range(nrStreams)]
    azimuths = azimuths + [azimuth -180 for azimuth in azimuths]    #  nrStreams equally-spaced locations on a virtual circle (doesn't include depth)
    interAzimuthDistance = 360.0/len (azimuths) # the distance in degrees between azimuths (doesn't include depth)
    maxAzimuthVar = (2*interAzimuthDistance)-interAzimuthDistance    # the max variation in location from a given azimuth for a 'voice'/instrument
#    loc=choice ([0, 22.5, 45, 67.5, 90, 110.5, 135, 157.5, 180, -157.5, -135,-110.5, -90, -67.5, -45, -22.5])
    freq = 1 # just in case it slips through without definition
    for t in xrange(len (s.playList)):                      # s.playList (t) is the current event
                                                            # ===------- DEFAULTS -------===
        durat=0.165 #1; 1.2;0.17  # reset for each packet
        amp = 0.95 # was 1.2 after 0.25                                    # === AMPLITUDE ===
        atkT = 0.15 ; decT = 0 ; susL = 1                       # === ENVELOPE ===


        timeOffset = 0 # 15 # delay seconds before starting    # === TIME OFFSETS ===
        thisAttackTime = startTimes [t]
        timeIncrement = thisAttackTime - lastAttackTime
        if int(timeIncrement *250)/250 != 0:
            continue # i.e. abort this we only want a maximum of 1 event every nth of a sec
        # ===  the step ladder /VU meter on density represented as changes in pitch
        if  timeIncrement > 0.08: # arbitrary delay
            thisScaleNote = 1
        elif timeIncrement <= (1*deltaBeat): #* 1.5
             thisScaleNote = lastScaleNote +1 #+ choice([-3, -2, -1, -2, 0, 1]) # play the next scale note
        elif timeIncrement > (1*deltaBeat)  :#* 1.5
            thisScaleNote = max (0,lastScaleNote-1)
        else:  # 2 * - for speed
            thisScaleNote = lastScaleNote
        if thisScaleNote > 36:  # was 19 and then 61
            thisScaleNote = choice([2,3,4,5])
        freq = ( fund * thisScaleNote)
        freq = note2cps(thisScaleNote+24, fund, 8.0)
        lastScaleNote = thisScaleNote    # prepare for next event NB this gets reset at each buffer read
        lastAttackTime = thisAttackTime  # prepare for next event NB this gets reset at each buffer read

                # === IP ADDRESSES & PORTS ===
        ipAddrSrc = s.playList[t][s.packetFieldsDict['ipAdrSrc']]
        ipAddrDst = s.playList[t][s.packetFieldsDict['ipAdrDst']]
        tcpPortSrc = s.playList[t][s.packetFieldsDict['tcpIPportSrc']]
        tcpPortDst = s.playList[t][s.packetFieldsDict['tcpIPportDst']]
        udpPortSrc = s.playList[t][s.packetFieldsDict['udpIPportSrc']]
        if udpPortSrc == '':  # sometimes the UPD port is a null string
            udpPortSrc = -1
        udpPortDst = s.playList[t][s.packetFieldsDict['udpIPportDst']]
        if udpPortDst == '':  # sometimes the UPD port is a null string
            udpPortDst = -1

        srcIPname = s.IPaddrID(ipAddrSrc) # use the name string (NB: several IP addresses resolve to the same name)
        dstIPname = s.IPaddrID(ipAddrDst) # use the name string (NB: several IP addresses resolve to the same name)
        if TRACE:
            # print manyColons + "packetFieldsDict" ; print s.packetFieldsDict
            print manyColons+ " SRC & DEST IP names :", srcIPname, dstIPname
            print manyColons+ "TCP PORTS: src,dst + UDP PORTS: src,dst. -1 means aNullString: ", tcpPortSrc , tcpPortDst, udpPortSrc, udpPortSrc

        if IFSTATS:
            s.updateStatsDict(s.playList[t]) #  update the stats dict      print s.sflowStatsDict"""

                # === EVENT/INSTRUMENT ALLOCATIONS ====
        instrNr =  0           # default instrument: silence, do nothing (used during event filtering)
        azimuth= choice([0,45,90]) + ( maxAzimuthVar * random())
        tonic1 = 1920  # (8 * 60)
        amp *= 0.4
        tonic2 = tonic1 * 2 ** (1/12)     # whole tone
#        scale1 = [tonic1, tonic1 * 2 ** (1/6.0) ,tonic1 * 2 ** (2/6.0),tonic1 * 2 ** (3/6.0),tonic1 * 2 ** (4/6.0),tonic1 * 2 ** (5/6.0) ,tonic1 * 2]
#        print UIsettings['f']
    #       Firstly, drop almost all server-to-server activity
        if ('Server:' in srcIPname) and ('Server:' in dstIPname):
            if UIsettings['s'][0]=='S': # include all server-to-server comms...
                instrNr = 1
        if ('Server:Files' in srcIPname) and ('Server:Files' in dstIPname):
            if UIsettings['f'][0]=='F': # include all server-to-server comms...
                instrNr = 15
            else:
                instrNr = 0 # if we don't want file servers
        if 'Printer' in srcIPname or 'Printer' in dstIPname:
            if UIsettings['p'][0]=='p':                             # dst is another server, so nullify the srcIPname
                instrNr =  0  # redundant but best to make it clear
            else:                   # UIsettings['p'][0]=='P' i.e. we want the printers                                           # are a single string in the for 4eg "Server:Printer,Server:P"
                instrNr = 6
                if '10.54.12' in ipAddrDst:
                  azimuth = 0
                  freq = 1440
                elif '10.54.13' in ipAddrDst:
                  azimuth = 90
                  freq = 1620
                elif '10.54.14' in ipAddrDst:
                  azimuth = 180
                  freq = 1800
                elif '10.54.15' in ipAddrDst:
                  azimuth = 270
                  freq = 2160
                else:
                  azimuth = 360
                  freq = 720
                durat = durat * 5
                amp *= 0.45
# instr20 alternative
#                instrNr = 20
#                freq = 1 * choice ([7/8.0, 1, 9/8.0,10/9.0])
#                iSoundNr = 48 # choice ([39, 40, 41, 42]) # fingerSnapClosed
        #       Next, nullify all other server IDs
#                elif ('Server:' in srcIPname) and ('Server:Printer' not in srcIPname):    # if the src is any server but the printer server
#                     srcIPname = NULL_IP_NAME                                             # nullify it
#                elif ('Server:' in dstIPname) and ('Server:Printer' not in dstIPname):    # if the dst is any server but the printer server
#                     dstIPname = NULL_IP_NAME                                             # nullify it

#       Next, identify all _unidentifiedIP src AND dst

        elif ( '_unidentifiedIP' in srcIPname ):
            if UIsettings['u'][0]=='U':
                # external communications: treat for distance
                distanceRatio = ipDistanceRatio(ipAddrSrc)
                if TRACE:
                    print manyColons + " UNIDENTIFIED SOURCE. ip distance ratio: " , distanceRatio
                instrNr = 9  # glis sine
                azimuth= (360 * random())-180
                freq = scale1a[0]
                glisfreq = freq - max (0, (2 * freq * distanceRatio))  # in case we run into -ve freqs
                durat *= 2 * distanceRatio  # added * distanceRatio 21050119 to try to emulate length of tone rel to distance
                amp *= 0.35
        elif ('_unidentifiedIP' in dstIPname):
            if UIsettings['u'][0]=='U':
                # external communications: treat for distance
                distanceRatio = ipDistanceRatio(ipAddrDst)
                if TRACE:
                    print manyColons + "UNIDENTIFIED DESTINATION. ip distance ratio: ", distanceRatio
                instrNr = 9  # glis sine
                azimuth= (360 * random())-180
                freq = scale1a[1]
                glisfreq = freq +  max (0, (2 * freq * distanceRatio))  # in case we run into -ve freqs
                durat *= 2 * distanceRatio  # added * distanceRatio 21050119 to try to emulate length of tone rel to distance
                amp *= 0.35
        if ( '_unidentifiedIP' in srcIPname ) and ('_unidentifiedIP' in dstIPname):  # unidentified 2 way traffic - sound a gong!
             instrNr = 6  # fmbell
             azimuth = 360
             freq = 180
             durat = 5

#              azimuth= (360 * random())-180
#              freq= choice ([1653, 1860, 2092]) # (60 * 31 ... 1740 = 60 * 29
        if UIsettings['r'][0]=='R':
            if ('Department:Audio' in srcIPname):
                instrNr = 20
                #durat *= 2.1
                #freq = scale2[6] /2.0
                amp *= 0.06
                freq = 8
                durat +=1
                iSoundNr = choice ([20, 21, 22])  # fingerSnapClosed

            if ('Department:Audio' in dstIPname):
                instrNr = 20  #  voice        azimuth= -135 + ( maxAzimuthVar * random())
                #durat *= 2.1
                amp  *= 0.06
                freq = 7
                durat +=1
                iSoundNr = choice ([39, 40, 41, 42]) # fingerSnapClosed

            if ('Department:BS+BT' in srcIPname):
                instrNr = 20
                #durat *= 2.1
                #freq = scale2[6] /2.0
                amp *= 0.5
                freq = 5
                durat +=1
                iSoundNr = choice ([57, 58, 59]) # shakerOpen
            if ('Department:BS+BT' in dstIPname):
                instrNr = 20  #  voice        azimuth= -135 + ( maxAzimuthVar * random())
                #durat *= 2.1
                amp  *= 0.5
                freq = 4
                durat +=1
                iSoundNr = choice ([60, 61, 62]) # shakerShort

            if ('Department:HighFreq' in srcIPname):
                instrNr = 20
                #durat *= 2.1
                #freq = scale2[6] /2.0
                amp *= 0.5
                freq = 7
                durat +=1
                iSoundNr = choice ([63,64]) # # shaker1, 2
            if ('Department:HighFreq' in dstIPname):
                instrNr = 20  #  voice        azimuth= -135 + ( maxAzimuthVar * random())
                #durat *= 2.1
                amp  *= 0.5
                freq = 6
                durat +=1
                iSoundNr = choice ([64,65]) # shaker2, 3

        if UIsettings['i'][0]=='I':
            if ('ITgroup:' in srcIPname):
                instrNr = 20
                #durat *= 2.1
                #freq = scale2[6] /2.0
                amp *= 0.5
                freq = 1
                durat +=5
                iSoundNr = choice ([67]) # tambourine[67,68]
            if ('ITgroup:' in dstIPname):
                instrNr = 20  #  voice        azimuth= -135 + ( maxAzimuthVar * random())
                #durat *= 2.1
                amp  *= 0.5
                freq = 1
                durat +=5
                iSoundNr = choice ([72]) # tambourine [69, 70,71,72]

        if ('VOIP:' in srcIPname):
            if UIsettings['v'][0]=='V':
                instrNr = 20
                #durat *= 2.1
                #freq = scale2[6] /2.0
                amp *= 0.5
                freq = 1
                durat +=5
                iSoundNr = 76 # choice ([67,68]) # tom1-L
        if ('VOIP:' in dstIPname):
            if UIsettings['v'][0]=='V':
                instrNr = 20  #  voice        azimuth= -135 + ( maxAzimuthVar * random())
                #durat *= 2.1
                amp  *= 0.5
                freq = 1
                durat +=5
                iSoundNr = 77 # choice ([69, 70,71,72]) # tom2-LL

        if UIsettings['a'][0]=='A':        # all destinations unknown
            instrNr = 9
            distanceRatio = ipDistanceRatio(ipAddrSrc)
            glisfreq = freq - max (0, (4 * freq * distanceRatio))  # in case we run into -ve freqs
            azimuth= 180 + ( maxAzimuthVar * random())

        if UIsettings['l'][0]=='L':        # Load balancing sonification
            if ('_unidentifiedIP' in  dstIPname) or ('_unidentifiedIP'in  srcIPname):
                pass  # don't oerwrite the gliss tones
            else:
                instrNr = 15
                azimuth= 180 + ( maxAzimuthVar * random())

        if UIsettings['f'][0]=='f':        # shut up the file servers
            if ('Files' in dstIPname) or ('Files' in srcIPname):
                instrNr = 0

        if UIsettings['z'][0]=='Z':        # shut everything up
            instrNr = 0

# :::::::::::::::::::::   SOLO switch..... Drop all but the required ::::::::::::::::::::::::::#
#        if instrNr != 10:  instrNr  = 0       #instrNr = 15 # alrternative
#        if instrNr != 9 or instrNr != 6 or instrNr != 20:
#        if (instrNr != 6) and (instrNr != 9) and (instrNr != 20):
#           instrNr = 0
# :::::::::::::::::::::   End of sonification switching logic     :::::::::::::::::::::::::::::#


# :::::::::::::::::::::   Address Indentity Masking - for RTScore  ::::::::::::::::::::::::::::#
        srcIPname = srcIPname.split(',')    # split srcIPname name string into ['longName', 'shortName']
        dstIPname = dstIPname.split(',')
        if LEGEND_IDENTITY_MASK:            # thisfunctions globally. For individual masks, need to do it at the makeScore level
            srcIPname = srcIPname[-1]   # so if the split results in a single string, it will be returned
            dstIPname = dstIPname[-1]   # drw: expected this if-then logic to be reversed - no idea why!
        else:
            srcIPname = srcIPname[0]
            dstIPname = dstIPname[0]

        srcScoreLabel = "\"" +srcIPname+ "\""   # necessary for csound string handling opcodes
        dstScoreLabel = "\"" +dstIPname+ "\""
# :::::::::::::::::::::   End: Address Indentity Masking   ::::::::::::::::::::::::::::::::::::#

# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::#
# :::::::::::::::::::::   End of sonification switching logic     :::::::::::::::::::::::::::::#
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::#
#
#for UDP send - clone for all instrs
#   Stringy1  sprintf ",%i,%5.2f,%i,%5.2f,%5.2f,%5.2f", ieventNr, istartTime, instrNr,ifreq1, ifreq2, idur
#   Stringy2  sprintf ",%s, %s", ScoreLabelSRC, ScoreLabelDST
#   Stringy strcat Stringy1, Stringy2
#   OSCsend  p1, gSudpDemoRoom, giudpDemoPort,"", "s", Stringy  ; UDP to demoRoom

                    # === GENERATE THE EVENT RECORDS AND EMIT FORSOUND RENDERING ===
        startTime = startTimes[t]+timeOffset
        if instrNr == 0:
            if TRACE:
                print manyColons + "NOT SYNTHESIZED: instrNr: %s dstScoreLabel: %s" % (instrNr, dstScoreLabel)

        if instrNr == 1:     # simpleSine
           if not WEB_UI or instrumentDict['01'][instrumentIsActiveField]: # [1] is On or OFF flag
 #       instrNr =  1 # choice ([ 6, 7  ]) #2    #instrNr = selectInstrument(s.playList, record)
#                makeCSevent ('i', 1, startTimes[t]+timeOffset, durat, amp * choice(expandMultMedRnge), freq, 0.04,  0.02,  0.5* durat, 0.1*durat, azimuth) # ; +filename )
                srcEvent = ['i', 1, startTimes[t]+timeOffset, durat, amp, freq,    0.04, durat-0.05,  0, 0.01, azimuth, OP_EVENT_COUNT, dstScoreLabel, srcScoreLabel]
                dstEvent = srcEvent[:]                  # make a copy not a reference to

                OP_EVENT_COUNT += 1
                makeCSevent ('i', 1, startTime, durat, amp, freq,    0.04, durat-0.05,  0, 0.01, azimuth, OP_EVENT_COUNT, dstScoreLabel, srcScoreLabel)
                if USE_LOCAL_IP:
                    sendUDP (",s",i,OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat-0.05, srcScoreLabel, dstScoreLabel, )  # transforms eventStr into a UPD message and emits it

        elif instrNr == 2:   # pluck
           if not WEB_UI or instrumentDict['02'][instrumentIsActiveField]: # [instrumentIsActiveField] True/False is On or OFF flag
                method = 1 #choice ([1, 3])  # 1 3 4 are possible
                mParam1 =  1 # choice ([0.8, 0.9, 1])
                mParam2 =  10 # choice([1, 10])
                #azimuth= 90 + ( maxAzimuthVar * random())
                #amp = amp * choice ([0.5, 0.8])
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 2, sstartTime, durat,  amp* 0.4, freq, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel ) #ipNamesString) #'; '+filename )
#                makeCSevent ('i', 2, startTimes[t]+timeOffset, durat,  amp* 0.4*choice(reduceMultSmlRnge), freq, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 3:  # preppno
           if not WEB_UI or instrumentDict['03'][instrumentIsActiveField]: # [1] is On or OFF flag
                method = 4
                mParam1 =  0.5#choice ([0.8, 0.9, 1])
                mParam2 =  1 #choice([1, 10])
                #azimuth= 0 + ( maxAzimuthVar * random())
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 2, startTime, durat*1.5,  amp, freq*2, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
#                makeCSevent ('i', 2, startTimes[t]+timeOffset, durat*1.5,  amp* choice(reduceMultMedRnge), freq*2, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, dstScoreLabel) #ipNamesString) #'; '+filename )
            # preppno
     #          #makeCSevent ('i', 3, sttime, durat,              ifreq,  iNS, iD,    iK,  iT30, iHFloss, kclampL, kclampR    iHmass	, iHfreq, iHinit, iSpos, ivel,isfreq, isspread irattles,irubbers]) #'; '+filename )
     #           makeCSevent ('i', 3, startTimes[t], durat,     freq,    3, 10,    1,     5,   0.002,    2,      2,        1,        5000,    -1,	 0.09,  amp,   1,     0.01,     31,        32,azimuth) # ; +filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 4:  #  hammond
           if not WEB_UI or instrumentDict['04'][instrumentIsActiveField]: # [1] is On or OFF flag
                azimuth= -135 + ( maxAzimuthVar * random())
                #azimuth= 90 + ( maxAzimuthVar * random())
                #makeCSevent ('i', 4, sttime,         idur, kamp, kfreq, kmodI, kxfade,kvibR ,kvibD, ivibED, kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 4, startTime,   durat*1.4, amp*0.5,  freq,   0.5,    0.5,  13,    0.05 , 0.01,  azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString)#'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 5:  #   cheby polys
           if not WEB_UI or instrumentDict['05'][instrumentIsActiveField]: # [1] is On or OFF flag
                #makeCSevent ('i', 5, sttime,       idur,                   iamp, kfreq, kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 5, startTime, durat*0.5,   amp*0.5,     freq, azimuth, OP_EVENT_COUNT, dstScoreLabel, srcScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 6:  # fm bell
           if not WEB_UI or instrumentDict['06'][instrumentIsActiveField]: # [1] is On or OFF flag
                #azimuth= -180 + ( maxAzimuthVar * random())
                #makeCSevent ('i', 6, sttime,                  idur, kamp, kfreq, kmodI, kxfade,  kvibD, kvibR ,kdecR, kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 6, startTime, durat, amp* 0.4,  freq,   0.5,    1.0,   0.005 , 3,   4, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 7:   # fm voice
           if not WEB_UI or instrumentDict['07'][instrumentIsActiveField]: # [1] is On or OFF flag
#               azimuth= -90 + ( maxAzimuthVar * random())  ; vowel = choice(range(65)) # used for testing only because they are not defined before this instr is selected
               tilt = choice (range (90))
#               makeCSevent ('i', 7, sttime,                    idur,   kamp,       kfreq,    6kvowel 7ktilt 8kvibD, 9kvibR 10kazimuth) #'; '+filename )
               OP_EVENT_COUNT += 1
               makeCSevent ('i', 7, startTime, durat,   amp*0.6,     freq*0.1,   vowel, tilt,   0.005 , 6,   azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
               if USE_LOCAL_IP:
                   sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 8:   # highhat
           if not WEB_UI or instrumentDict['08'][instrumentIsActiveField]: # [1] is On or OFF flag
                #makeCSevent ('i', 8, sttime,                    idur,         kamp,   10kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 8, startTime, durat*.4,   amp*0.1,  azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
#                makeCSevent ('i', 8, startTimes[t]+timeOffset, durat*.4,   amp*0.1 * choice(rduceMultMedRnge),  azimuth, OP_EVENT_COUNT, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 9:   # glisInstr
           if not WEB_UI or instrumentDict['09'][instrumentIsActiveField]: # i.e. the 'can be played' flag in the instrumentDict is True
               # NB this specifically is not controlled by "if WEB_UI" so that it can be set manually when it is first initialised.
#               if UI_CONTROL_LIST[0]=='3':  # long live the birds!
#                   if UI_CONTROL_LIST[1] == '1':
               freq *= 2
               glisfreq = max(30, 4 * glisfreq * choice([1,-1]))
#                   if UI_CONTROL_LIST[2] == '1':
#                       freq *= 1.5 # up a 5th !
#                       amp *= 0.618
#                   if UI_CONTROL_LIST[3] == '1':
               azimuth = choice (azimuths)
#               else:
#                   freq = 2 * freq
#                          'i', 9,     p1instrNr   p2startTime     p3durat, p4amp,                          p5freq p6atk  p7sust p8release  p9release p10glisfreq, p11azimuth
#               makeCSevent ('i', 9, startTimes[t]+timeOffset, durat, amp, freq, 0.04,  0.02,  0.5* durat, 0.1*durat, glisfreq, azimuth) # ; +filename )
#               glisfreq = 100 ; azimuth = 0   # used for testing only because they are not defined before this instr is selected
               OP_EVENT_COUNT += 1
               makeCSevent ('i', 9, startTime, durat, amp*0.6, freq,   0.04, durat-0.05,  0, 0.01, glisfreq, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) # ; +filename )
               if USE_LOCAL_IP:
                   sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 10:    # fractalNoise
             if not WEB_UI or instrumentDict['10'][instrumentIsActiveField]: # [1] is On or OFF flag
                freq = 0.2 * choice ([1440, 1800]) #choice ([1620, 1800, 2160])
          #             #i11    2st    3idur 4iamp  5inrBeads 6iFBenrgy 7kazimuth
                #makeCSevent ('i', 10, sttime,                  idur, iamp, ifreq,    ibwd, iattak, isust, azimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 10, startTime, durat, amp*0.25, freq,  0.05, 0.01, 0.1,   azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 11:    # crunch - not used - substitute Nr 10 instead
             if not WEB_UI or instrumentDict['11'][instrumentIsActiveField]: # [1] is On or OFF flag
          #             #i11    2st    3idur 4iamp  5inrBeads 6iFBenrgy 7kazimuth
                #makeCSevent ('i', 11, sttime,                    idur,     iamp,   inrBeads, iFBenrgy kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 11, startTime, durat,  amp*0.15,    2,        0.04,  azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 12:    # guiro
           if not WEB_UI or instrumentDict['12'][instrumentIsActiveField]: # [1] is On or OFF flag
                #makeCSevent ('i', 12, sttime,                  idur, iamp,    iifreq1, iifreq2 inrBeads  kazimuth, OP_EVENT_COUNT, "scoreLable ) #'; '+filename )
                pass
 #               dstScoreLabel = "\"Koala Fleisch 12\""
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 12, startTime, 0.3,   amp*0.06,  2500,  4000,  7,       azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString)#'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 13:    # bamboo
           if not WEB_UI or instrumentDict['13'][instrumentIsActiveField]: # [1] is On or OFF flag
                #makeCSevent ('i', 13, sttime,                   idur, kamp, 5inrBeads    idamp ifreq1, iifreq2 iifreq3  kazimuth) #'; '+filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 13, startTime, 0.01, amp*0.2,   1.05,     0.05,     2800,  2240,  3360,   azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 14:    # tambourine
           if not WEB_UI or instrumentDict['14'][1]: # [1] is On or OFF flag                            2300,  5600,  8100
                 #makeCSevent ('i', 14, sttime,                kamp,  idur,  5inrBeads idamp ifb     ifreq1, iifreq2 iifreq3  kazimuth) #'; '+filename )
               OP_EVENT_COUNT += 1
#               dstScoreLabel = "\"Koala Fleisch 14\""
               makeCSevent ('i', 14, startTime, amp *0.2,  .02,    32,   0.47,   0 ,      1150,  2800,  4050,   azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) #'; '+filename )
               if USE_LOCAL_IP:
                   sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 15:
            if not WEB_UI or instrumentDict['15'][instrumentIsActiveField]: # [1] is On or OFF flag
           # PulseFMsynth_15
            #makeCSevent ('i', 15, sttime,                   idur,  iamp,
                wobbleFactor = 0.2
#                             carrfreq  p6iattk, p7isus,p8idky,    p9irel    p10imodamp, p11imodfreq [[2+(10*random())]], p12imode, p13imodeArg, p14azimuth
#                makeCSevent ('i', 15, startTimes[t]+timeOffset, durat, (amp) * choice(expandMultMedRnge), freq,  0.04,  0.02,  0.5* durat, 0.1*durat,   freq*(2+wobbleFactor),  (1.0/durat)*(2*random()),  10,   0.5, azimuth) # ; +filename )
                OP_EVENT_COUNT += 1
                makeCSevent ('i', 15, startTime, durat, amp*0.35, freq,  0.04, durat-0.05,  0, 0.01,   freq*(2+wobbleFactor),  (1.0/durat)*(2*random()),  10,       0.5,   azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel) #ipNamesString) # ; +filename )
#                makeCSevent ('i', 15, startTimes[t]+timeOffset, durat, (amp) * choice(expandMultMedRnge), freq,  0.04, durat-0.05,  0, 0.01,   freq*(2+wobbleFactor),  (1.0/durat)*(2*random()),  10,       0.5,   azimuth, OP_EVENT_COUNT, dstScoreLabel) #ipNamesString) # ; +filename )
                if USE_LOCAL_IP:
                    sendUDP (",s", OP_EVENT_COUNT, startTime, instrNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

#                                            p2                   p3      p4    p5       p6         p7       p8              p9              p10
        elif instrNr == 20:  # test Instrument
            if not WEB_UI or instrumentDict['20'][instrumentIsActiveField]: # [1] is On or OFF flag

                OP_EVENT_COUNT += 1
#                                            p2                   p3      p4    p5       p6         p7       p8              p9              p10
#                print 'i', 20, startTimes[t]+timeOffset, durat,   amp , freq,  iSoundNr,   azimuth, OP_EVENT_COUNT, srcScoreLabel,  dstScoreLabel
                makeCSevent ('i', 20, startTime, durat,   amp , freq,  iSoundNr,   azimuth, OP_EVENT_COUNT, srcScoreLabel,  dstScoreLabel)
                if USE_LOCAL_IP: # NB for just this instrument we substitute the iSoundNr for the Instrument number
                    sendUDP (",s", OP_EVENT_COUNT, startTime, iSoundNr, freq, freq, durat,   srcScoreLabel, dstScoreLabel)  # transforms eventStr into a UPD message and emits it

        elif instrNr == 99:
            gSudpInet= UI_CONTROL_LIST[-1]  # the output UDP port address, as a string
            makeCSevent ('i', 99, startTime, OP_EVENT_COUNT, "\""+gSudpInet+"\"") #ipNamesString) # ; +filename )
#        if (t % 1000) == 0:
#            time.sleep(5)


#"""=========================  SFLOW DATA CONTROL ==========================="""
s = SflowSonifyer(dataDirectory) # where the fifo and pipe filling is done. Set at top of this file
s.loadActiveIPaddrs()
s.loadPortIDs()
s.printRate = 1000000 ; s.printORnot=False

if TRACE:
# IPdict contains entries like ('10.55.68', '10.55.69', '10.55.70', '10.55.71'): 'EZRT_Departments'
    print manyColons + "IPdict" ; print s.IPdict
    print manyColons + "IPdict.values()" ; print s.IPdict.values()
    print manyColons + "portsDict" ; print  s.portsDict
    print manyColons + "portsDict.values()" ; print s.portsDict.values()

# Identify some fixed IP addreesses
ServErlangenFiles    = '153.96.171'
ServErlangen         = '153.96.170.0'
ServDNSname2IPaddr   = '153.96.170.12'

"""
print s.IPdict.values()
print s.IPdict.keys()
#print s.portsDict.keys()

#s.initStatsDict()  # initialises the sflow stats collection dictionary - doesn't work - yet - but the update function does the job - almost!
#print s.sflowStatsDict
# the above init doesn't work becasue of the IPdict keys are tuples

#s.readSflowFile2RecordList()  # THIS ONE

#s.extractSingleRecordFromFile(1001)
#print s.unidentifiedIPaddr
#print s.suspectIPaddr
#print s.unidentifiedPorts

#s.makePlayList()    # THIS ONE # for i in range (len(s.playList)): print s.playList[i]
"""

                                   # p2      p3     p4    p5   p6      p7      p8       p9    p10   p11   p12
"""====================  SFLOW SOUND RENDERING CONTROL  ======================="""
import csnd6
cs=csnd6.Csound()
cs.SetOption('-odac')
cs.SetOption('-b 4096')
csPerf =csnd6.CsoundPerformanceThread(cs)       # instantiate the RT perf thread
cs.ReadScore("f 0 z \n")                        # instantiate CS
# sound "fonts" for rendering"
cs.ReadScore("f 20 0 0 1 \"../../sounds/agogo1-H.wav\" 0 0 0 \n")
cs.ReadScore("f 21 0 0 1 \"../../sounds/agogo2-M.wav\" 0 0 0 \n")
cs.ReadScore("f 22 0 0 1 \"../../sounds/agogo3-L.wav\" 0 0 0 \n")
cs.ReadScore("f 23 0 0 1 \"../../sounds/bell-cow1-H.wav\" 0 0 0 \n")
cs.ReadScore("f 24 0 0 1 \"../../sounds/bell-cow2-M.wav\" 0 0 0 \n")
cs.ReadScore("f 25 0 0 1 \"../../sounds/bell-cow3-L.wav\" 0 0 0 \n")
cs.ReadScore("f 26 0 0 1 \"../../sounds/bell-glass1.wav\" 0 0 0 \n")
cs.ReadScore("f 27 0 0 1 \"../../sounds/bell-glass2.wav\" 0 0 0 \n")
cs.ReadScore("f 28 0 0 1 \"../../sounds/bells-cow-jangly-L.wav\" 0 0 0 \n")
cs.ReadScore("f 29 0 0 1 \"../../sounds/bongo.wav\" 0 0 0 \n")
cs.ReadScore("f 30 0 0 1 \"../../sounds/cabasa.wav\" 0 0 0 \n")
cs.ReadScore("f 31 0 0 1 \"../../sounds/clave.wav\" 0 0 0 \n")
cs.ReadScore("f 32 0 0 1 \"../../sounds/conga_clean1_HH.wav\" 0 0 0 \n")
cs.ReadScore("f 33 0 0 1 \"../../sounds/conga_clean2_H.wav\" 0 0 0 \n")
cs.ReadScore("f 34 0 0 1 \"../../sounds/conga_clean3_M.wav\" 0 0 0 \n")
cs.ReadScore("f 35 0 0 1 \"../../sounds/conga_clean4_L.wav\" 0 0 0 \n")
cs.ReadScore("f 36 0 0 1 \"../../sounds/conga_clean5_LL.wav\" 0 0 0 \n")
cs.ReadScore("f 37 0 0 1 \"../../sounds/conga_clean6_LLL.wav\" 0 0 0 \n")
cs.ReadScore("f 38 0 0 1 \"../../sounds/elctroperc.wav\" 0 0 0 \n")
cs.ReadScore("f 39 0 0 1 \"../../sounds/fingerSnapHard1_high.wav\" 0 0 0 \n")
cs.ReadScore("f 40 0 0 1 \"../../sounds/fingerSnapHard2_mid.wav\" 0 0 0 \n")
cs.ReadScore("f 41 0 0 1 \"../../sounds/fingerSnapHard3_mid.wav\" 0 0 0 \n")
cs.ReadScore("f 42 0 0 1 \"../../sounds/fingerSnapHard4_low.wav\" 0 0 0 \n")
cs.ReadScore("f 43 0 0 1 \"../../sounds/fingerSnapSoft1_high.wav\" 0 0 0 \n")
cs.ReadScore("f 44 0 0 1 \"../../sounds/fingerSnapSoft2_mid.wav\" 0 0 0 \n")
cs.ReadScore("f 45 0 0 1 \"../../sounds/fingerSnapSoft3_low.wav\" 0 0 0 \n")
cs.ReadScore("f 46 0 0 1 \"../../sounds/guiroH-L.wav\" 0 0 0 \n")
cs.ReadScore("f 47 0 0 1 \"../../sounds/guiroL-H.wav\" 0 0 0 \n")
cs.ReadScore("f 48 0 0 1 \"../../sounds/housewhistle.wav\" 0 0 0 \n")
cs.ReadScore("f 49 0 0 1 \"../../sounds/maraca.wav\" 0 0 0 \n")
cs.ReadScore("f 50 0 0 1 \"../../sounds/rattle1.aif\" 0 0 0 \n")
cs.ReadScore("f 51 0 0 1 \"../../sounds/rattle2.aif\" 0 0 0 \n")
cs.ReadScore("f 52 0 0 1 \"../../sounds/rattle3.aif\" 0 0 0 \n")
cs.ReadScore("f 53 0 0 1 \"../../sounds/rs1.wav\" 0 0 0 \n")
cs.ReadScore("f 54 0 0 1 \"../../sounds/rs3.wav\" 0 0 0 \n")
cs.ReadScore("f 55 0 0 1 \"../../sounds/rs5.wav\" 0 0 0 \n")
cs.ReadScore("f 56 0 0 1 \"../../sounds/rs6.wav\" 0 0 0 \n")
cs.ReadScore("f 57 0 0 1 \"../../sounds/shaker-open1.wav\" 0 0 0 \n")
cs.ReadScore("f 58 0 0 1 \"../../sounds/shaker-open2.wav\" 0 0 0 \n")
cs.ReadScore("f 59 0 0 1 \"../../sounds/shaker-open3.wav\" 0 0 0 \n")
cs.ReadScore("f 60 0 0 1 \"../../sounds/shaker-short1.wav\" 0 0 0 \n")
cs.ReadScore("f 61 0 0 1 \"../../sounds/shaker-short2.wav\" 0 0 0 \n")
cs.ReadScore("f 62 0 0 1 \"../../sounds/shaker-short3.wav\" 0 0 0 \n")
cs.ReadScore("f 63 0 0 1 \"../../sounds/shaker1.wav\" 0 0 0 \n")
cs.ReadScore("f 64 0 0 1 \"../../sounds/shaker2.wav\" 0 0 0 \n")
cs.ReadScore("f 65 0 0 1 \"../../sounds/shaker3.wav\" 0 0 0 \n")
cs.ReadScore("f 66 0 0 1 \"../../sounds/shirr.wav\" 0 0 0 \n")
cs.ReadScore("f 67 0 0 1 \"../../sounds/tambourine-openH.wav\" 0 0 0 \n")
cs.ReadScore("f 68 0 0 1 \"../../sounds/tambourine-openL.wav\" 0 0 0 \n")
cs.ReadScore("f 69 0 0 1 \"../../sounds/tambourine1-H.wav\" 0 0 0 \n")
cs.ReadScore("f 70 0 0 1 \"../../sounds/tambourine2-M.wav\" 0 0 0 \n")
cs.ReadScore("f 71 0 0 1 \"../../sounds/tambourine3-L.wav\" 0 0 0 \n")
cs.ReadScore("f 72 0 0 1 \"../../sounds/tambourine4-LL.wav\" 0 0 0 \n")
cs.ReadScore("f 73 0 0 1 \"../../sounds/timbale1-H.wav\" 0 0 0 \n")
cs.ReadScore("f 74 0 0 1 \"../../sounds/timbale3-L.wav\" 0 0 0 \n")
cs.ReadScore("f 75 0 0 1 \"../../sounds/timbaleNoise-M.wav\" 0 0 0 \n")
cs.ReadScore("f 76 0 0 1 \"../../sounds/tom1-L.wav\" 0 0 0 \n")
cs.ReadScore("f 77 0 0 1 \"../../sounds/tom2-LL.wav\" 0 0 0 \n")
cs.ReadScore("f 78 0 0 1 \"../../sounds/triangle-bright.wav\" 0 0 0 \n")
cs.ReadScore("f 79 0 0 1 \"../../sounds/triangle-mute.wav\" 0 0 0 \n")
cs.ReadScore("f 80 0 0 1 \"../../sounds/triangle-open.wav\" 0 0 0 \n")
cs.ReadScore("f 81 0 0 1 \"../../sounds/woodblock-hi.wav\" 0 0 0 \n")
cs.ReadScore("f 82 0 0 1 \"../../sounds/woodblock.wav\" 0 0 0 \n")

# end soundFonts for rendering

cs.CompileOrc(sflowOrchestraOptions)            # SR, NrChans etc       # from sflowOrchestra.py

cs.Start()                                      # start CS synth
csPerf.Play()

cs.CompileOrc(sineInstr_01)     # compile an instrument  # from sflowOrchestra.py these (for eg sineInstr_01) are variable names
cs.CompileOrc(pluckInstr_02)    # compile an instrument  # from sflowOrchestra.py
cs.CompileOrc(preppno_03)
cs.CompileOrc(fmb3_04)
cs.CompileOrc(cheby_05)
cs.CompileOrc(fmbell_06)
cs.CompileOrc(fmvoice_07)
cs.CompileOrc(highhat_08)
cs.CompileOrc(glisInstr_09)
cs.CompileOrc(fractalNoise_10)
cs.CompileOrc(crunch_11)
cs.CompileOrc(guiro_12)

cs.CompileOrc(bamboo_13)
cs.CompileOrc(tambourine_14)
cs.CompileOrc(pulseFMsynth_15)
cs.CompileOrc(testInstrument_20)

cs.CompileOrc(setUDPout_99)         # used to initialise the upd IP address and port number
cs.ReadScore("f 1 0 16384 10 1")
cs.ReadScore("f 2 0 17 0 0 1 1")
cs.ReadScore("f31 0 8 2 1 0.6 10 100 0.001 ; 1 rattle")
cs.ReadScore("f32 0 8 2 1 0.7 50 500 1000  ; 1 rubber")

# to suppress ALL terminal output from csound
def passCallback():  pass
if not TRACE:
    cs.SetMessageCallback(passCallback())


#cs.Stop() this, combined with the starting & stopping in makeEvent() works -not sure iy is better though
#  :::::::::::::::::::::::::::::::: USER INTERFACE CONTROL VIA WEB :::::::::::::::::::::::::::::
jsonUIcontrolFile       = 'jsonUIcontrols.txt' # controls to this file generated by # http://www2.iis.fraunhofer.de/corpo/controls.php
jsonUIcontrolFileBackup = jsonUIcontrolFile+'_bu'

import os
import shutil
def readUIcontrolsFromWeb():
    global UI_CONTROL_LIST                  # initialised at top of file or just above
    sock = urllib.urlopen(UIcontrolURL)     # open the URL for reading the controls - WHAT ABOUT open ("xxx", 'r')
    UI_CONTROL_LIST = sock.readlines()      # read the UI control file, generated at the URL by
    sock.close()
    UI_CONTROL_LIST = UI_CONTROL_LIST[0].rstrip().split(",")
    print manyColons + "UI_CONTROL_LIST: ", UI_CONTROL_LIST

def readUIcontrolsFromFile():
    global FILE_UI_FID # assumes fid is open for reading
    if FILE_UI:
        dicky=json.load(FILE_UI_FID)
        return dicky

def readFifoUI():
    global FILE_UI_FID # assumes fid is open for reading
    if FILE_UI:
        dicky=json.load(FILE_UI_FID)
        return dicky

def printFifoUI():
#    global FILE_UI_FID
    global UIsettings
    kees = UIsettings.keys()
    kees.sort()
    print "_____________________________\nNetSon single char UI Control\n-----------------------------"
    for kee in kees:
        if UIsettings[kee][0]!= kee:
            print '*',
        print UIsettings[kee]


def doInner (sflowStream):
    global IFSTATS
    global WEB_UI
    global FILE_UI
    global UIsettings
    jsonSflowStatsFile="jsonSflowStats.txt"
    nrpkts=0
#    while 1:                                   # now replaced by the GUI mainloop()
    sflowStream.readStream ()
    nrpkts += sflowStream.nrPacketsToRead
    s.playList = sflowStream.payloadPackets
    sflowStream.payloadPackets = []             # having copied the payload to s.playList, zero it.
    if WEB_UI:
        try:
            readUIcontrolsFromWeb()
            print UI_CONTROL_LIST
        except:
            print " Unable to fetch UI control data from" + UIcontrolURL
            print UI_CONTROL_LIST

    if FILE_UI:
        try:
            UIsettings = readUIcontrolsFromFile()
            printFifoUI()

        except:
            pass
            #print " Unable to fetch UIsettings (UI control data from FILE_UI_FID)"

    playAllInstruments()  # commenting out this line will cause no sound to be generated
#        time.sleep(sflowStream.sleepTime) # not needed -  control handed to GUI
    if IFSTATS:
        jsonSflowFP = open(jsonSflowStatsFile, 'w')
        json.dump(s.sflowStatsDict, jsonSflowFP) # json dumps the (almost) raw pull data to file. Time of write should be encoded in the filename
        jsonSflowFP.flush()
        jsonSflowFP.close()
        shutil.copy2(jsonSflowStatsFile, jsonSflowStatsFile+'_bu') # copy so read/write don't have to wait for each other. a fix?
        os.remove(jsonSflowStatsFile)
        jsonSflowFP = 0

def doit(csoundInstance):
#    global FILE_UI
#    global FILE_UI_FID
    liveStream = SflowPipeRead(dataDirectory)
    liveStream.delay            = 5000      # ms btwn pkt timestamp and sonification
    liveStream.sleepTime        = 0.001     # min nr of seconds between pipe reads
    liveStream.nrPacketsToRead  = 50        # nr pf packets to read from the pipe in one hit was 400
    liveStream.logLiveDataQ     = False     # collect streamed data for later testing if needed.
    liveStream.clearOldPipeData()           # clears out the Pipe to receive new (current) packets
    if liveStream.logLiveDataQ:
        liveStream.livePipeLogFD = open (liveStream.livePipeLogFilename, 'r')
    return liveStream                        # added to enable GUI to be set up properly

#    doInner(liveStream)                 # contains the while 1 loop
#    csoundInstance.Stop()                       # stops Csound
#    csoundInstance.Cleanup()                    # clean up Csound; this is useful if you're going to reuse a Csound instance

if not TRACE:
    print "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    print "Ready to start the sonification from the user interface."
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++"


x=doit(cs)
while 1:
#    runUI()
    doInner(x)
    break


if __name__ == "__main__":
    doit(cs)
    while 1:
        doInner(x)

"""
print "::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: Ho HO HO - GOT HERE 0 ::::::::::::::::"

#GLOBAL DISTANCE RATIO CODE - USING A PUBLIC DATABASE
drw: this doesn't work properly because not all IP addresses are resolved to physical locations by the DB.

import pygeoip  # for resolving ip addresses to geolocations # http://www.movable-type.co.uk/scripts/latlong.html
geoIPCityDB = pygeoip.GeoIP('../GeoLiteCity.dat') # for usage, see https://code.google.com/p/pygeoip/wiki/Usage

def globalDistanceRatio(homelat, homelong, ipaddrString):
    # uses law of cosine approximtion: Law of cosines: d = acos( sin φ1 ⋅ sin φ2 + cos φ1 ⋅ cos φ2 ⋅ cos Δλ ) ⋅ R
    ipaddrCity = geoIPCityDB.record_by_addr(ipaddrString)  # get the city location of the IP address
    if ipaddrCity == None:
        ipLongInt = ip2long(ipaddrString)
        # print manyColons + " Unidentified IIS address : %s addr: %sdiff: %s" %(IISipLongInt, ipLongInt, abs(IISipLongInt-ipLongInt))
        return ip2long(ipaddrString)
    else:
        lat2 = radians(float(ipaddrCity['latitude']))     # convert its location to radians
        long2 = radians(float(ipaddrCity['longitude']))   # convert its location to radians
        dlat = lat2 - homelat
        dlong = long2 - homelong
        a = (sin(dlat/2))**2 + cos(homelat) * cos(lat2) * (sin(dlong/2))**2
        res = meanEarthRadius * 2 * atan2(sqrt(a), sqrt(1-a))
        print "DISTANCE FROM ERLANGEN: %.4f " % res
        return  res/halfEquatorLength


def globalDistanceRatio(homelat, homelong, ipaddrString):
    ipLongInt = ip2long(ipaddrString)
    # print manyColons + " Unidentified IIS address : %s addr: %sdiff: %s" %(IISipLongInt, ipLongInt, abs(IISipLongInt-ipLongInt))
    return ip2long(ipaddrString)

#    return 6373.0 *      # distance in kms;  6373.0 is mean radius of the earth
# globalDistanceRatio (latErlangen,longErlangen, '153.96.171.0')
"""
